@@ -23,6 +23,7 @@ sys.path.insert(0, os.path.abspath('../..'))
 extensions = [
     'sphinx.ext.autodoc',
     'sphinx.ext.extlinks',
+    'sphinxcontrib.apidoc',
     #'sphinx.ext.intersphinx',
     'openstackdocstheme',
     'oslo_config.sphinxext',
@@ -51,7 +52,7 @@ exclude_patterns = ['api/oslo_policy.tests.*', 'api/setup.rst']
 # General information about the project.
 project = u'oslo.policy'
 copyright = u'2014, OpenStack Foundation'
-source_tree = 'https://git.openstack.org/cgit/openstack/oslo.policy/tree'
+source_tree = 'https://opendev.org/openstack/oslo.policy/src/branch/master'
 
 # If true, '()' will be appended to :func: etc. cross-reference text.
 add_function_parentheses = True
@@ -95,3 +96,7 @@ extlinks = {
 
 # Example configuration for intersphinx: refer to the Python standard library.
 #intersphinx_mapping = {'http://docs.python.org/': None}
+
+# -- sphinxcontrib.apidoc configuration --------------------------------------
+apidoc_module_dir = '../../oslo_policy'
+apidoc_output_dir = 'reference/api'
